Skip to contents

an extension to greta with functions for simulating dynamical systems, defined by of ordinary differential equations (see ode_solve()) or transition matrices (iterate_matrix()).

Author

Maintainer: Nicholas Tierney nicholas.tierney@gmail.com (ORCID)

Authors: